/*
 * $Id: internal-connections.c,v 0.10 1992/05/18 23:09:21 ceder Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
 * internal-connections.c
 *
 * Abstract routines on the data type Connection.
 */

static char *rcsid = "$Id: internal-connections.c,v 0.10 1992/05/18 23:09:21 ceder Exp $";


#include "exp.h"
#include <kom-types.h>
#include <server/smalloc.h>
#include "com.h"
#include "connections.h"
#include "internal-connections.h"
#include "lyskomd.h"
#include "config.h"
#include "log.h"

INTERNAL  Connection  *all_connections   = NULL;
INTERNAL  Session_no   no_of_connection_attempts = 0;

/* Used in get_conn_by_number to speed things up. */

INTERNAL  Connection  *last_conn = NULL;

INTERNAL const Connection EMPTY_CONNECTION =
    ((Connection){0, NULL, NULL,
		      /* Things used by services.c */
		      0, NULL, NO_TIME, 0,
		      EMPTY_STRING_i, 0,
		      EMPTY_STRING_i, EMPTY_STRING_i, EMPTY_STRING_i,
		      FALSE, FALSE,
		      /* Used by server/connections.c */
		      NULL, '\0',
		      0, 0, 0, 0, 0, 0, 0,
		      /* Gather data... */
		      0, 0, 0, 0, EMPTY_STRING_i, EMPTY_STRING_i,
		      EMPTY_STRING_i, NULL, NULL, DEFAULT_PRIV_BITS_i,
		      NULL_CONF_TYPE_i, EMPTY_tm_i,
		      /* Protocol independent... */
		      EMPTY_STRING_i, 0, FALSE,
		      NO_TIME, 0});

static int  no_of_allocated_connections = 0;

INTERNAL Connection *
alloc_connection(void)
{
    Connection *res;
    
    res = smalloc ( sizeof(Connection) );
    *res = EMPTY_CONNECTION;
    ++no_of_allocated_connections;
    return res;
}

EXPORT  Connection *
new_client(void)
{
    Connection		*c;

    c = alloc_connection();

    c->magic = CONN_MAGIC_ALLOC;
    
    if (all_connections != NULL)
    {
	all_connections->prev->next = c;
	c->prev = all_connections->prev;
	all_connections->prev = c;
	c->next = all_connections;
    }
    else
    {
	c->prev = c;
	c->next = c;
    }

    all_connections = c;
    
    c->session_no = ++no_of_connection_attempts;
    c->protocol = '\0';      /* Reserved to mean 'no protocol specified yet' */
    c->unparsed = EMPTY_STRING;
    c->first_to_parse = 0;
    c->more_to_parse = TRUE;
    c->last_request = NO_TIME;
    c->pers_no = 0;
    c->person = NULL;
    time( & c->session_start );
    c->cwc = 0;
    c->what_am_i_doing = EMPTY_STRING;
    c->ena_level = 0;
    c->username = EMPTY_STRING;
    c->ident_user = EMPTY_STRING;
    c->invisible = FALSE;
    c->username_valid = FALSE;

    return c;
}

/*
 * Only used from logout_client
 */
EXPORT  void
kill_client(Connection *cp)    
{
    if ( all_connections == NULL )
	restart_kom("kill_client(): No clients in all_connections list\n");
    
    if (all_connections == all_connections->next)
    {
	if (all_connections->prev != all_connections)
	    restart_kom("kill_client(): all_connections corrupt (LINK)\n");

	if (all_connections != cp)
	    restart_kom("kill_client(): all_connections corrupt (SINGLE)\n");
      
	all_connections = NULL;
    }
    else
    {
	if (cp->prev == NULL || cp->next == NULL)
	    restart_kom("kill_client(): all_connections corrupt (NULL)\n");
      
	cp->prev->next = cp->next;
	cp->next->prev = cp->prev;
	
	if (all_connections == cp)
	    all_connections = cp->next;
    }

    if ( last_conn == cp )
	last_conn = NULL;

    cp->magic = CONN_MAGIC_FREE;

    s_clear(&cp->unparsed);
    s_clear(&cp->what_am_i_doing);
    s_clear(&cp->username);
    s_clear(&cp->hostname);
    s_clear(&cp->ident_user);

    if ( cp->mux != NULL )
    {
	log("kill_client(): client %d has mux != NULL.\n",
	    cp->session_no);
    }

    if ( !s_empty(cp->c_string0) || !s_empty(cp->c_string1)
	|| !s_empty(cp->string0) )
    {
	log("kill_client(): unexpected string remains.\n");
    }

    if ( cp->c_misc_info_p != NULL || cp->c_local_text_no_p != NULL )
	log("kill_client(): unexpected remaining data.\n");

    sfree(cp);
    --no_of_allocated_connections;
}

#ifdef DEFENSIVE_CHECKS

INTERNAL  void
check_conn_exists(Connection *foo)
{
    /* Check that foo really is active. */

    Connection *c = all_connections;
    Bool found = FALSE;

    do
    {
	if ( c == foo )
	    found = TRUE;

	c = c->next;
    }
    while ( c != all_connections && found == FALSE );

    if ( found == FALSE )
	restart_kom("get_conn_by_number: foo == %d not found in"
		    " all_connections.\n", last_conn->session_no);
}

#endif

/*
 * Session_nos must NOT be recycled, or this code might break!
 */
EXPORT  Connection *
get_conn_by_number (Session_no session_no)
{
    Connection *end;

    if ( last_conn == NULL )
	last_conn = all_connections;
    else if ( all_connections == NULL )
    {
	restart_kom("get_conn_by_number(%d): last_conn = %d and "
		    "all_connections == NULL", session_no,
		    last_conn->session_no);
    }
#ifdef DEFENSIVE_CHECKS
    else
	check_conn_exists(last_conn);
#endif

    end = last_conn;

    do
    {
	if ( last_conn->session_no == session_no )
	    return last_conn;

	last_conn = last_conn->next;
    }
    while ( last_conn != end);

    return NULL;
}


EXPORT  Session_no
traverse_connections (Session_no session_no)
{
    Connection *prev = NULL;

    if ( all_connections == NULL )
	return 0;

    prev = get_conn_by_number ( session_no );

    /* prev is NULL if session_no is 0, or if session_no was logged out. */

    if ( prev == NULL )
	return all_connections->session_no;
    else if ( prev->next == all_connections )
	return 0;		/* Full circle. */
    else
	return prev->next->session_no;
}

void
dump_allocated_connections(FILE *fp)
{
    fprintf(fp, "---" __FILE__ ":\n\tConnections:      %d\n",
	    no_of_allocated_connections);
    fprintf(fp, "\tConnection attempts:	%d\n",
	    no_of_connection_attempts);
}
